#include "HighScores.h"

/**
 * @brief HighScores ctor, loads scores from file if it exists
 * @param x distance from left side of screen
 * @param y distance from top of screen
 * @param w width of entries
 */
HighScores::HighScores(unsigned x, unsigned y, unsigned w)
    :x(x), y(y), w(w), h(50) {
        
    std::ifstream file_in("scores.dat");
    
    if(file_in.is_open()) {
        unsigned temp;
        while(file_in >> temp) {
            add(temp);
        }
    }
    
    sort();
}

/**
 * @brief HighScores dtor, saves scores to file upon closing
 * @return 
 */
HighScores::~HighScores() {
    std::ofstream file_out("scores.dat");
    for(unsigned i = 0; i < scores.size() && i < max_scores; i++) {
        file_out << scores[i] << " ";
    }
    file_out << std::endl;
}

/**
 * @brief Add a score to the highscores
 * @param new_score the newly achieved score to add
 */
void HighScores::add(unsigned new_score) {
    /* If the maximum number of scores hasn't been reached yet */
    if(scores.size() < max_scores) {
        scores.push_back(new_score);
    }
    /* If the new score is better than the current worst */
    else if(scores.back() < new_score) {
        scores.back() = new_score;
    }
    sort();
}

/**
 * @brief Sorts the scores in descending order
 */
void HighScores::sort() {
    std::sort(scores.begin(), scores.end(), std::greater<unsigned>());
}

/**
 * @brief Displays the scores on the screen using the global renderer
 */
void HighScores::draw() const {
    unsigned current_y = y;
    for(auto s : scores) {
        /* Background */
        boxColor(sdlRenderer, x, current_y, x+w, current_y+h, 0x3494BAFF);
        /* Text */
        centeredtextRGB(sdlRenderer, x+w/2, current_y+h/2, h*.75, std::to_string(s), 0x94, 0xE1, 0xFF);
        current_y += h+20;
    }
}

/**
 * @brief Displays scores until Back button is activated
 */
void HighScores::loop() {
    /* Back button creation */
    Menu hs_menu(100, y + scores.size()*(h+20), 340);
    hs_menu.add_button(SDLK_b, "Back", "B");
    
    /* Draw contents on screen, commit changes */
    draw();
    hs_menu.draw();
    SDL_RenderPresent(sdlRenderer);

    bool quit = false; /* Quit flag */
    while(SDL_WaitEvent(&ev) && !quit) { /* Regular event loop for Menu */
        int command = -1;
        switch(ev.type) {
            case SDL_QUIT:
                quit = true;
                break;
            case SDL_MOUSEBUTTONDOWN:
                if(ev.button.button == SDL_BUTTON_LEFT) {
                    command = hs_menu.click(ev.button.x, ev.button.y);
                }
                break;
            case SDL_KEYDOWN:
                command = hs_menu.press(ev.key.keysym.sym);
                break;
        }
        if(command == 0) {
            quit = true;
        }
    }
}
